# Eiko Fried, Webinar for the Transcontinental Computational Psychiatry Workgroup
# June 12th, 2018

# More information: www.eiko-fried.com, Twitter @EikoFried, eikofried@gmail.com
# All materials for the Webinar at: www.eiko-fried.com/TCPW2018 
# The dataset we use for the webinar is from DOI 10.1016/j.janxdis.2016.11.008

# If you're looking for more free data to practice with, see: http://eiko-fried.com/data/



############### MRF 1: network estimation and inference

# ---------------------------------------------------------------------------------------
# ---------- 1. Load libaries -----------------------------------------------------------
# ---------------------------------------------------------------------------------------

library("qgraph")
library("bootnet") 
library("dplyr")

# ---------------------------------------------------------------------------------------
# ---------- 2. Adjacency Matrix --------------------------------------------------------
# ---------------------------------------------------------------------------------------

AdjacencyMatrix <- matrix(0,4,4)  # create matrix
AdjacencyMatrix[1,2] <- AdjacencyMatrix[2,1] <- AdjacencyMatrix[2,3] <- AdjacencyMatrix[3,2] <- AdjacencyMatrix[2,4] <- AdjacencyMatrix[4,2] <- 1  # fill it with values
AdjacencyMatrix 
qgraph(AdjacencyMatrix) # plot network from values of the matrix using 'qgraph' function of 'qgraph' package

# ---------------------------------------------------------------------------------------
# ---------- 3. Data preparation --------------------------------------------------------
# ---------------------------------------------------------------------------------------

# set working directly to where the data file is; data from DOI 10.1016/j.janxdis.2016.11.008
load("data.Rdata")
View(data)

# the following 3 lines are to make the plots a bit fancier (to add a legend etc), you don't need this. 
names<-c("B1","B2", "B3", "B4", "B5", "C1", "C2", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "E1", "E2", "E3", "E4", "E5", "E6")
longnames <- c("Intrusive thoughts", "Nightmares", "Flashbacks", "Emotional cue reactivity", "Physiological cue reactivity", "Avoidance of thoughts", "Avoidance of reminders", "Trauma-related amnesia", "Negative beliefs", "Blame of self or others", "Negative trauma-related emotions", "Loss of interest", "Detachment", "Restricted affect", "Irritability/anger", "Self-destructive/reckless behavior", "Hypervigilance", "Exaggerated startle response", "Difficulty concentrating", "Sleep disturbance")
gr <- list("Intrusions"=c(1:5), "Avoidance"=c(6:7), "Cognition & mood alterations"=c(8:14),"Arousal & reactivity alterations"=c(15:20)) 

# ---------------------------------------------------------------------------------------
# ---------- 4. Correlation network -----------------------------------------------------
# ---------------------------------------------------------------------------------------

corMat <- cor_auto(data) # estimate appropriate correlations among items; in our case, these are polychoric

# sometimes there are problems with polychoric correlations, in which case you want to use Spearman via cor(data, method="spearman")
# for details on this see discussion in https://arxiv.org/abs/1607.01367 (accepted in Psychological Methods)

View(corMat)

g1 <- qgraph(corMat, layout="circle") # visualize correlation matrix
g1 <- qgraph(corMat, layout="circle", details=TRUE) # visualize correlation matrix
g1 <- qgraph(corMat, layout="circle", cut=0) # visualize correlation matrix

# add some graphical arguments defined in the beginning of the script, and plot it as PDF

pdf("correlation_network.pdf", width=7, height=5)
g1b <-qgraph(corMat, labels=names, layout="circle", vsize=6, border.width=1.5, border.color="black", 
             groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.35, cut=0)
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 5. Fruchtermann-Reingold algorithm -----------------------------------------
# ---------------------------------------------------------------------------------------

corMat2 <- corMat

pdf("compare0.pdf", width=7, height=5)
g1c <-qgraph(corMat2, labels=names, layout="spring", vsize=6, border.width=1.5, border.color="black", 
             groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.35, cut=0)
dev.off()

# now we change the correlation matrix (the network input) slightly
corMat2[1,2] <- corMat2[2,1] <- corMat[1,2] + 0.0001 # we change the correlation matrix just a tiny little bit

# the visual result is very different now

pdf("compare1.pdf", width=8, height=5)
layout(t(1:2))
g1d <-qgraph(corMat, layout="spring", vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), cut=0)
g1e <-qgraph(corMat2,layout="spring", vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), cut=0)
dev.off()

# the algorithm is chaotic, and the tiniest changes to the input matrix will lead to fairly different layouts
# we therefore constrain the layout to be equal

L1 <- averageLayout(corMat, corMat2)

pdf("compare2.pdf", width=8, height=5)
layout(t(1:2))
g1d <-qgraph(corMat, layout=L1, vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), cut=0)
g1e <-qgraph(corMat2,layout=L1, vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), cut=0)
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 6. Visually compare networks -----------------------------------------------
# ---------------------------------------------------------------------------------------

### Obtain a correlation matrix with much smaller correlations
corMat3 <- corMat/3

pdf("compare3.pdf", width=8, height=5)
layout(t(1:2))
g1f <-qgraph(corMat, layout=L1, cut=0, details=TRUE, title="normal network")
g1g <-qgraph(corMat3, layout=L1, cut=0, details=TRUE, title="network with smaller edges") # surprisingly, networks look the same ... because of scaling (R "zooms in" and scales everything accordingly)
dev.off()                                             # the details=TRUE command helps us to see what the maximum edge is

pdf("compare4.pdf", width=8, height=5)
layout(t(1:2))
g1j <-qgraph(corMat, layout=L1, details=T, cut=0, maximum=0.74) # and setting the maximum allows us visual comparison
g1jk <-qgraph(corMat3, layout=L1, details=T, cut=0, maximum=0.74)
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 7. Partial correlation network ---------------------------------------------
# ---------------------------------------------------------------------------------------

# so far: visualization of correlations; now: network models

pcor <- estimateNetwork(data, default="pcor") # estimateNetwork function from package 'bootnet'
dev.off()

pdf("pcor.pdf")
pcor_plot <- plot(pcor, details=T, layout=L1, cut=0) # we obtain spurious negative relationships; estimateNetwork by default uses theme='colorblind'
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 8. Regularized partial correlation network ---------------------------------
# ---------------------------------------------------------------------------------------

GGM <- estimateNetwork(data, default="EBICglasso") # lasso regularized gaussian graphical model for ordinal or gaussian data

pdf("GGM.pdf", width=9, height=5)
GGM_plot<-plot(GGM, labels=names, layout="spring", vsize=6, cut=0, border.width=1.5, border.color="black", 
         groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.4)
dev.off()

# assume we wanted Spearman correlations instead: 

GGM2 <- estimateNetwork(data, default="EBICglasso", corMethod = "cor", 
                        corArgs=list(method = "spearman", use = "pairwise.complete.obs"))
GGM_plot2<-plot(GGM2, labels=names, layout="spring", vsize=6, cut=0, border.width=1.5, border.color="black", 
               groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.4)

layout(t(1:2))
plot(GGM, layout=L1)
plot(GGM2, layout=L1)
dev.off()

# how similar are these 2 networks? correlate adjacency matrixes!
cor(GGM$graph[upper.tri(GGM$graph)], GGM2$graph[upper.tri(GGM2$graph)], method='spearman')

# let's look at the adjacency matrix (going back to GGM, polychoric correlations)

adjacencyMatrix <- GGM$graph
View(adjacencyMatrix)

# plot all 3 together
pdf("compare6.pdf", width=8, height=4)
layout(t(1:3))
qgraph(corMat, layout=L1, cut=0, details=T, title="Cor, Pcor, and GGM") # correlation network
qgraph(pcor_plot, layout=L1, details=T, cut=0, theme="classic") # partial correlation network
plot(GGM, layout=L1, details=T, cut=0, theme="classic") # GGM
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 9. Ising Model -------------------------------------------------------------
# ---------------------------------------------------------------------------------------

data_bin <- apply(data, 2, function(x) ifelse(x > 1, 1, 0)) # we dichotomize data so we have an example case of binary data; we do this just to learn how to fit an Ising Model 
dev.off()

IsingModel <- estimateNetwork(data_bin, default="IsingFit") # Ising Model
IsingModel_plot<-plot(IsingModel, labels=names, layout=L1, vsize=6, cut=0, border.width=1.5, border.color="black", 
               groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.4)

pdf("compare7.pdf", width=8, height=5)
layout(t(1:2))
plot(GGM, layout=L1, details=T, cut=0, title="GGM") # GGM
plot(IsingModel, layout=L1, details=T, cut=0, title="Ising Model") # Ising Model; edges not standardized.
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 10. Network inference -------------------------------------------------------
# ---------------------------------------------------------------------------------------

pdf("centrality.pdf")
c1 <- centralityPlot(GGM) # estimate and plot centrality estimates for GGM network
dev.off()

# expected influence
c2 <- centralityPlot(GGM, include=c("Strength", "ExpectedInfluence")) 